#ifndef	_CONFIG_INC_
#define	_CONFIG_INC_ 1

; *** general configuration for AVR MCU
#ifndef F_CPU
#	define F_CPU (8000000)
#endif

#define BRRDIV	8
#define BRR(X)	( (F_CPU/BRRDIV/((X)/10)+5)/10 - 1)
#define	BRR_VAL	BRR(38400)

; *** general register assignment for application program
#define fcnt_1ms	r2
#define dcnt_1ms	r3
#define dcnt_1ms_1	r4 // for CLCD_wait
#define dcnt_1ms_2	r5 // for I2Ctimeout
#define dcnt_1ms_3	r6 // for I2C reset timer

#define dcnt_CLCD	dcnt_1ms_1

#define USI_STAT	r8
#define USI_ADDRW	r9
#define USI_DATA	r10
#define USI_TEMPADDRW	r11

#define USI_STAT_NotReady   	-1
#define USI_STAT_Idle       	0
#define USI_STAT_AddressRecv	1
#define USI_STAT_AddressACK 	2
#define USI_STAT_WaitForSend	3
#define USI_STAT_DataSend   	4
#define USI_STAT_DataCHK    	5
#define USI_STAT_DataRecv   	6
#define USI_STAT_DataACK     	7

#define EXT_LCD_STAT_CMD    	0
#define EXT_LCD_STAT_DAT    	1
#define EXT_LCD_STAT_CONT   	2
#define EXT_LCD_STAT_AKIFLAG   	4
#define EXT_LCD_STAT_15FLAG    	8


#define PREV_SW_STAT	r12
#define UART_inc	r13
#define EXT_LCD_STAT r14
#define ALT_I2C_EOR_MASK r15

#define CLCD_NO r16
;nouse r17
;nouse r18
;nouse r19

; r20-r25 are temporarie working registers
; r26-r31 are pointer registers

; *** event flags
#define	updateFLG	GPIOR0

#define	updateCNTms	7	// set if periodic count
#define	update_6	6	// unused
#define	updateI2CZ	5	// set if I2C reset force
#define	updateI2CI	4	// set if I2C set idel state
#define	updateI2CS	3	// set if I2C wait for send-request
#define	updateI2CR	2	// set if I2C in data-received
#define	updateSCI	1	// set if incoming data from UART
#define	updateSW	0	// set if change SW status

; *** LED&SWITCH_KEY flags
#define	LEDOUT  	GPIOR1
#define	SW_STAT 	GPIOR2


; *** I2C LCD definition
; port description for CLCD
; PD6/ICP   	CLCD:E(pin6)
; PD5/OC0B/T1	CLCD:RS(pin4)
; PD4/T0    	CLCD:RW(pin5)
; PD2/INT0/XCLK	SW_LED_EN#
; PB3..PB0  	CLCD:DB7..DB4(pin14..11)=BTNLED:4..1
; PB6       	CLCD2:E(if defined TWIN_LCD)

#define TWIN_LCD 1
#define LCD_IS_MULTI_LINES 1
#define USART_ON_ALT_LCD 1

// #define	I2C_LEDMATRIX

// #define	LEDMATRIX_IS_PRIMARY_LCD_PORT

#define SW_LED_PT	PORTD, PD2  // SW_LED_EN#
#define SW_LED_EN	SBI 	SW_LED_PT
#define SW_LED_DI	CBI 	SW_LED_PT

#define LCD_RW_PT	PORTD, PD4  // LCD_RW
#define LCD_RS_PT	PORTD, PD5  // LCD_RS

#define LCD_DB7_PT	PORTB, PB3  // LCD_DB7
#define LCD_DB6_PT	PORTB, PB2  // LCD_DB6
#define LCD_DB5_PT	PORTB, PB1  // LCD_DB5
#define LCD_DB4_PT	PORTB, PB0  // LCD_DB4

#ifndef TWIN_LCD
#	define LCD_E_PT	PORTD, PD6  // LCD_E
#	define LCD_E_EN	SBI 	LCD_E_PT
#	define LCD_E_DI	CBI 	LCD_E_PT

#else
#	define LCD_E1_PT	PORTD, PD6  // LCD_E1
#	define LCD_E2_PT	PORTB, PB6  // LCD_E2
#	ifndef	I2C_LEDMATRIX
.macro	LCD_E_EN
	SBRS	CLCD_NO,0
	SBI 	LCD_E1_PT
	SBRC	CLCD_NO,0
	SBI 	LCD_E2_PT
.endm
.macro	LCD_E_DI
	CBI 	LCD_E1_PT
	CBI 	LCD_E2_PT
.endm
#	else
#		ifdef LEDMATRIX_IS_PRIMARY_LCD_PORT
#			define LCD_E_PT 	LCD_E1_PT  // for LEDmatrix
#			define LCD_E_EN	SBI	LCD_E2_PT
#			define LCD_E_DI	CBI	LCD_E2_PT
#		else
#			define LCD_E_PT 	LCD_E2_PT  // for LEDmatrix
#			define LCD_E_EN	SBI	LCD_E1_PT
#			define LCD_E_DI	CBI	LCD_E1_PT
#		endif
#	endif

#endif /* TWIN_LCD */


; *** I2C slave address definition
#define	I2C_ADDR_BTNLED 0x52	//	; Native SW/LED ctrl
#define	I2C_ADDR_EEPROM 0x56	//	; 128B(0x00-0x80) EEPROM emulate.
#define	I2C_ADDR_AKILCD 0x50	//	; akiduki(0x50) compat.
#define	I2C_ADDR_LCDCMD 0x51	//	; Native CLCD CMDwrite/STATread
#define	I2C_ADDR_LCDDAT 0x53	//	; Native CLCD GxRAM write/read
#define	I2C_ADDR_15LCD  0x3E	//	; ichigo(0x3e)  compat.

#ifdef TWIN_LCD
#define	I2C_ADDR_AKILCD_ALT 0x54	//	; akiduki(0x50) compat.
#define	I2C_ADDR_LCDCMD_ALT 0x55	//	; Native CLCD CMDwrite/STATread
#define	I2C_ADDR_LCDDAT_ALT 0x57	//	; Native CLCD GxRAM write/read
#endif

#ifndef I2C_ADDR_EEPROM
#	undef	I2C_LEDMATRIX
#endif

#ifdef I2C_LEDMATRIX
#	ifdef TWIN_LCD
#		ifndef LEDMATRIX_IS_PRIMARY_LCD_PORT
#			undef	I2C_ADDR_AKILCD_ALT
#			undef	I2C_ADDR_LCDCMD_ALT
#			undef	I2C_ADDR_LCDDAT_ALT
#		else
#			undef	I2C_ADDR_AKILCD
#			undef	I2C_ADDR_LCDCMD
#			undef	I2C_ADDR_LCDDAT
#			undef	I2C_ADDR_15LCD
#		endif
#	else
#		undef	I2C_ADDR_AKILCD
#		undef	I2C_ADDR_LCDCMD
#		undef	I2C_ADDR_LCDDAT
#		undef	I2C_ADDR_15LCD
#	endif
#endif


; *** I2C LEDMATRIX_feature definition
#ifdef I2C_LEDMATRIX
#	define LEDMATRIX_STROB_H	SBI 	LCD_RW_PT
#	define LEDMATRIX_STROB_L	CBI 	LCD_RW_PT
#	define LEDMATRIX_LATCH_H	SBI 	LCD_E_PT
#	define LEDMATRIX_LATCH_L	CBI 	LCD_E_PT
#	define LEDMATRIX_CLOCK_H	SBI 	LCD_DB7_PT
#	define LEDMATRIX_CLOCK_L	CBI 	LCD_DB7_PT
#	define LEDMATRIX_SIN1_H	SBI 	LCD_DB4_PT
#	define LEDMATRIX_SIN1_L	CBI 	LCD_DB4_PT
#	define LEDMATRIX_SIN2_H	SBI 	LCD_DB5_PT
#	define LEDMATRIX_SIN2_L	CBI 	LCD_DB5_PT
#	define LEDMATRIX_SIN3_H	SBI 	LCD_DB6_PT
#	define LEDMATRIX_SIN3_L	CBI 	LCD_DB6_PT
.macro	CLEAR_PORT reg
	CLI
	IN  	\reg, PORTB
	ANDI	\reg, 0xf0
	OUT 	PORTB, \reg
	SEI
.endm
.macro	LEDMATRIX_ENA
	SW_LED_DI               ; /* disable SW_LED  */
;	LEDMATRIX_STROB_H       ; /* set     LCD_R/W=H (negate /STROB)  */
	RCALL	_BUS_WR         ; // r25
	IN  	r24, PORTB      ; /* get  Port-B ODR */
	PUSH	r24             ; /* save Port-B ODR */
.endm
.macro	LEDMATRIX_DIS
	POP 	r24             ; /* load    Port-B ODR */
	ANDI	r24, 0x0f
	CLI
	IN  	r25, PORTB
	ANDI	r25, 0xf0
	OR  	r24, r25
	OUT 	PORTB, r24      ; /* restore Port-B ODR */
	SEI
	LEDMATRIX_STROB_L       ; /* set     LCD_R/W=L (assert /STROB)  */
	SW_LED_EN               ; /* enable  SW_LED  */
.endm
#	define LEDMATRIX_BITMAP_INVERSE_ORDER
#endif

; *** for debugging

#define putchar	_SCI_putchr

// #define I2C_BUS_FAULT_CHECK

#endif	/* _CONFIG_INC_ */
