; i2c_s.S
;  I2C slave feature support(w/clock stretching)
; refer to http://yesavr.web.fc2.com/tiny2313/AVR_I2C_IO/AVR_I2C_IO.html
;
;  USI(TWI)
;   USICR:=
;   	USISIE   = 1 := enable
;   	USIOIE   = 1 := enable
;   	USIWM1,0 = 0b11 := TWI w/clock line suspention
;   	USICS1.0 = 0b10 := external clock source(slave) ,shift w/rise edge
;   	USICS1.0 = 0b11 := external clock source(slave) ,shift w/fall edge
;   	USICLK   = 0 := only external clock strobe
;   	USITC    = 0 := no toggle SCL
;

#include	"common.inc"

#define USI_SETCOUNT_14     	0xe0 // (14<<4)

#define USI_SDA_IN  	CBI 	DDRB,5
#define USI_SDA_OUT 	SBI 	DDRB,5
#define USI_SDA_LOW 	CBI 	PORTB,5
#define USI_SDA_FREE	SBI 	PORTB,5
#define USI_SCL_LOW 	CBI 	PORTB,7
#define USI_SCL_FREE	SBI 	PORTB,7

#define DISOVF	CBI 	USICR,6
#define ENAOVF	SBI 	USICR,6

;	*****************  CONST *****************
.section	.eeprom
.align	0

;	*****************  COMM  *****************
.section	.bss
.align	0

;	*****************  TEXT  *****************
.section	.text
.align	1, 0xff

; /*******************************************************************/
;  INTERRUPT HANDLER (I2C BUS START CONDITION)
BEGIN	USI_START	;; I2C bus caught start condition handler(50clks=22clks + USI_SETSTATE_FLAGCLR(42clks))
	IN  	save_sreg, SREG         ; 1
	PUSH	r24                     ; 2
	PUSH	r25                     ; 2

	MOV 	r25, USI_STAT           ; 1
	COM 	r25                     ; 1
	BREQ	9f                      ; 1/2

	USI_SDA_FREE                    ; 2
	USI_SDA_IN                      ; 2
	LDI 	r24, USI_STAT_AddressRecv ;1
	MOV 	USI_STAT, r24           ; 1

	LDI 	r24, 0xe0               ; 1 // clear all FLAGS cnt=0
	OUT 	USISR, r24              ; 1

9:
	POP 	r25                     ; 2
	POP 	r24                     ; 2
	OUT 	SREG, save_sreg         ; 1

	RETI                            ; 4
ENDFN

; /*******************************************************************/
;  setup USI (must be set SCL(PB7) pin as output w/H_out, SDA(PB5) as input w/PU )
BEGIN	_INIT_USI	;; void INIT_USI(void); // r24,25
	CLI
	USI_SDA_IN                      ; 2
	USI_SDA_FREE                    ; 2
	USI_SCL_FREE                    ; 2
	LDI 	r24, 0xf8	; set USISIE, USIOIE, USIisI2C w/clockStretch, extCLK_riseEdge strobe
	OUT 	USICR, r24
	LDI 	r24, 0xe0               ; 1 // clear all FLAGS cnt=0
	OUT 	USISR, r24              ; 1
	LDI 	r24, USI_STAT_Idle      ; set Idle state
	MOV 	USI_STAT, r24
	CBI 	updateFLG, updateI2CS   ; 2
	CBI 	updateFLG, updateI2CR   ; 2
	CBI 	updateFLG, updateI2CI   ; 2
	CBI 	updateFLG, updateI2CZ   ; 2
	SEI
RETURN

BEGIN	_DEINIT_USI	;; void DEINIT_USI(void); // r24,25
;; deinitialize USI at invalid status sensed 
	LDI 	r24, 200
	MOV 	dcnt_1ms_2, r24
	CLR 	r24                     ; 1
	OUT 	USICR, r24              ; 1
	USI_SDA_IN                      ; 2
	USI_SDA_FREE                    ; 2
	USI_SCL_FREE                    ; 2
	SER 	r24                     ; 1
	MOV 	USI_STAT, r24           ; 1
	CBI 	updateFLG, updateI2CS   ; 2
	CBI 	updateFLG, updateI2CR   ; 2
RETURN

BEGIN	_IDLE_USI	;; void IDLE_USI(void); // r24,25
	CLI
	USI_SDA_IN                      ; 2
	USI_SDA_FREE                    ; 2
	USI_SCL_FREE                    ; 2
	CBI 	updateFLG, updateI2CS   ; 2
	CBI 	updateFLG, updateI2CR   ; 2
	CBI 	updateFLG, updateI2CI   ; 2
	LDI 	r24, 0xe0               ; 1 // clear all FLAGS cnt=0
	OUT 	USISR, r24              ; 1
	MOV 	r24, USI_STAT
	COM 	r24
	BREQ	9f
	CLR 	USI_STAT
9:
	SEI
RETURN

BEGIN	USI_stop_check
	SBIS	USISR, 5
	RET
	CBI 	updateFLG, updateI2CS   ; 2 // cancel reqSend
	LDI 	r24, 0xe0
	OUT 	USISR, r24
	CLR 	USI_STAT
RETURN

BEGIN	addressMatch
	MOV 	r25, r24                ; 1
	LSR 	r25                     ; 1
	EOR 	r25, ALT_I2C_EOR_MASK   ; 1

#ifdef	I2C_ADDR_LCDDAT
	CPI 	r25, I2C_ADDR_LCDDAT    ; 1
	BREQ 	9f                      ; 1/2
#endif

#ifdef	I2C_ADDR_LCDCMD
	CPI 	r25, I2C_ADDR_LCDCMD    ; 1
	BREQ 	9f                      ; 1/2
#endif

#ifdef	I2C_ADDR_BTNLED
	CPI 	r25, I2C_ADDR_BTNLED    ; 1
	BREQ 	9f                      ; 1/2
#endif

#ifdef	I2C_ADDR_EEPROM
	CPI 	r25, I2C_ADDR_EEPROM    ; 1
	BREQ 	9f                      ; 1/2
#endif

#ifdef	I2C_ADDR_AKILCD
	CPI 	r25, I2C_ADDR_AKILCD    ; 1
	BREQ 	9f                      ; 1/2
#endif

#ifdef	I2C_ADDR_15LCD
	CPI 	r25, I2C_ADDR_15LCD     ; 1
	BREQ 	9f                      ; 1/2
#endif


#ifdef	I2C_ADDR_LCDDAT_ALT
	CPI 	r25, I2C_ADDR_LCDDAT_ALT    ; 1
	BREQ 	9f                      ; 1/2
#endif

#ifdef	I2C_ADDR_LCDCMD_ALT
	CPI 	r25, I2C_ADDR_LCDCMD_ALT    ; 1
	BREQ 	9f                      ; 1/2
#endif

#ifdef	I2C_ADDR_AKILCD_ALT
	CPI 	r25, I2C_ADDR_AKILCD_ALT    ; 1
	BREQ 	9f                      ; 1/2
#endif

	CLR 	r24                     ; 1
9:
RETURN

; /*******************************************************************/
;  INTERRUPT HANDLER (I2C BUS CLOCK OVERFLOWED)
BEGIN	USI_OVF	;; I2C bus controller state counter overflow handler
	IN  	save_sreg, SREG         ; 1
	PUSH	r24                     ; 2
	PUSH	r25                     ; 2

	MOV 	r24, USI_STAT           ; 1
	CPI 	r24, USI_STAT_AddressRecv ;1
	BREQ	1f                      ; 1/2

	CPI 	r24, USI_STAT_AddressACK; 1
	BREQ	2f                      ; 1/2

	CPI 	r24, USI_STAT_DataSend  ; 1
	BREQ	3f                      ; 1/2

	CPI 	r24, USI_STAT_DataCHK   ; 1
	BREQ	4f                      ; 1/2

	CPI 	r24, USI_STAT_DataRecv  ; 1
	BREQ	5f                      ; 1/2

	CPI 	r24, USI_STAT_DataACK    ; 1
	BREQ	20f                      ; 1/2

	RJMP	7f                      ; 2
//	RCALL	_DEINIT_USI              ; 4 + n
//	RJMP	8f                      ; 2


1:	; // AddressRecv is done
	IN  	r24, USIDR              ; 1
	RCALL	addressMatch            ; 4 + n
	AND 	r24, r24                ; 1
	BREQ	7f                      ; 1/2

	CLR 	EXT_LCD_STAT            ; for aki & 15 LCD

	USI_SDA_OUT                     ; 2
	USI_SDA_LOW                     ; 2
	MOV 	USI_TEMPADDRW, r24      ; 1
//	RCALL	_SCI_BYTEOUT  ; //                  DEBUG line
	LDI 	r24, USI_STAT_AddressACK ;1
	LDI 	r25, 14                 ; 1
	RJMP	8f                      ; 2


2:	; // AddressACK is done
	USI_SDA_FREE                    ; 2
	LDI 	r24, 1                  ; 1
	AND 	r24, USI_TEMPADDRW      ; 1
	BREQ	20f // jump if R/W=L    ; 1/2
; if R/W=H
21:
		MOV 	USI_ADDRW, USI_TEMPADDRW ;1
		USI_SDA_OUT                     ; 2
		SBI 	updateFLG, updateI2CS   ; 2
		USI_SCL_LOW                     ; 2
		LDI 	r24, USI_STAT_DataSend  ; 1
		RJMP	29f

20: ; if R/W=L or DataACK  is done
		USI_SDA_FREE                    ; 2
		USI_SDA_IN                      ; 2
		SBIC	updateFLG, updateI2CR   ; 1/2,3
			USI_SCL_LOW                 ; 2
		SBIS	updateFLG, updateI2CR   ; 1/2,3
			MOV 	USI_ADDRW, USI_TEMPADDRW ;1
		LDI 	r24, USI_STAT_DataRecv  ; 1
29:
	LDI 	r25, 0                  ; 1
	RJMP	8f                      ; 2


5:	; // DataRecv is done
	SBI 	updateFLG, updateI2CR   ; 2
	IN  	USI_DATA, USIDR         ; 1
	USI_SDA_OUT                     ; 2
	USI_SDA_LOW                     ; 2
	LDI 	r24, USI_STAT_DataACK   ; 1
	LDI 	r25, 14                 ; 1
	RJMP	8f                      ; 2


3:	; // DataSend is done
	USI_SDA_IN                      ; 2
	LDI 	r24, USI_STAT_DataCHK   ; 1
	LDI 	r25, 14                 ; 1
	RJMP	8f
4:	; // DataCHK is done
	IN  	r24, USIDR              ; 1
	ANDI	r24, 1                  ; 1
	BREQ	21b ; // ACK received   : 1/2

; // to Idle state if NAK received
7:
	CLR 	r24
	CLR 	r25
8: ; // clear FLAGS and set cnt
	MOV 	USI_STAT, r24           ; 1
	ANDI	r25, 0x0f               ; 1
	ORI 	r25, 0xc0               ; 1 // clear S,OVF FLAGS & set cnt
	OUT 	USISR, r25              ; 1

//	RCALL	USI_stop_check

9: ; exit
	POP 	r25                     ; 2
	POP 	r24                     ; 2
	OUT 	SREG, save_sreg         ; 1
	RETI                            ; 4
ENDFN

; /*******************************************************************/
;  SEND DATA
BEGIN	_SEND_USI	;; void SEND_USI(uint8_t data); // r24-25
	MOV 	r25, USI_STAT           ; 1
	CPI 	r25, USI_STAT_DataSend  ; 1
	BRNE	8f

	OUT 	USIDR, r24              ; 1
	CBI 	updateFLG, updateI2CS   ; 2
	USI_SCL_FREE                    ; 2 // release clock stretch

8:
RETURN                              ; 4

; /*******************************************************************/
;  RECEIVE DATA
BEGIN	_RECV_USI	;; uint8_t RECV_USI(void); // r24-25
	MOV 	r25, USI_STAT           ; 1
	CPI 	r25, USI_STAT_DataRecv  ; 1
	BREQ	1f
	CPI 	r25, USI_STAT_DataACK   ; 1
	BRNE	8f
1:
	MOV 	USI_ADDRW, USI_TEMPADDRW ;1
	MOV 	r24, USI_DATA           ; 1

	CBI 	updateFLG, updateI2CR   ; 2
	USI_SCL_FREE                    ; 2 // release clock stretch

8:
RETURN                              ; 4

